\name{Ancestors}
\alias{ancGraph}
\alias{anGraph}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Ancestor graphs}
\description{
Compute the overall ancestors graph from a given DAG.
}
\usage{
ancGraph(A)
anGraph(A)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{A}{a square Boolean matrix representing
    the edge matrix of the DAG}
}
\details{
  The overall ancestor graph is a graph obtained from a DAG by
  computing the transitive closure. This graph has a non zero
  \eqn{(i,j]} entry iff node \eqn{j} is an ancestor of node \eqn{i}.

  \code{ancGraph} is based on a simple
  formula \eqn{(2 I - A)^{-1}} explained by Wermuth and Cox (2003).

  \code{anGraph} instead uses repeated composition applied to the original
  edge matrix. If \code{A} is not a DAG sometimes \code{ancGraph} fails, while
  \code{anGraph} is correct.
}
\value{
  both functions return the same result, a square Boolean matrix of the
  same order as \code{A} giving the edge matrix of the overall ancestor graph.
  
}
\references{
  Wermuth, N. \& Cox, D.R. (2003). Joint response graphs
  and separation induced by triangular systems. Submitted and available at
  \url{http://psystat.sowi.uni-mainz.de}.
}

\author{Giovanni M. Marchetti}
\seealso{\code{\link{DAG}}, \code{\link{is.acyclic}}}
\examples{
dag <- DAG(y ~ x, x ~ z+u,  u ~ v, a ~ v + b); dag
anGraph(dag) 
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{DAG}
\alias{DAG}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Defining directed acyclic graphs (DAGs)}
\description{
  A simple way to define a DAG by means of regression model
  formulae.
  }
\usage{
DAG(..., order = FALSE, test = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{\dots}{a sequence of model formulae}
  \item{order}{logical, defaulting to \code{FALSE}. If \code{TRUE} the nodes of the
    DAG are permuted to obtain and upper triangular edge matrix. If
    \code{FALSE} the nodes are in the order they first appear in the model
    formulae (from left to right).}
  \item{test}{logical value. If \code{TRUE} (the default) then it is
    tested if the graph is acyclic. Else if \code{test=FALSE} the
    test is bypassed. In this case the function may return a cyclic
    graph.}  
}
\details{
  The DAG is defined by a sequence of recursive regression models.
  Each regression is defined by a model formula.
  For each formula the response defines a node of the graph and
  the explanatory variables the parents of that node. If the
  regressions are not recursive the function returns an error message
  (if \code{test=TRUE}).
  
  The function returns the edge matrix of the graph, i.e. 
  a square Boolean matrix of order equal to the number of nodes of the
  graph and a one in position \eqn{(i,j)} if there is an arrow from
  \eqn{j} to \eqn{i} and zero otherwise. By default this matrix
  has ones along the main diagonal. Optionally (if \code{order = TRUE}) the
  edge matrix is permuted to be in upper triangular form (this can
  always be done for DAGs). The rownames of the edge matrix are
  the nodes of the DAG.
}
\value{a square Boolean matrix, with dimnames, the edge matrix of the DAG.}
\references{
  Lauritzen, S. (1996). \emph{Graphical models}. Oxford:
  Clarendon Press.
  
  Wermuth, N. \& Cox, D.R. (2003). Joint response graphs
  and separation induced by triangular systems. Submitted and available at
  \url{http://psystat.sowi.uni-mainz.de}.
}
\author{G. M. Marchetti}
\note{The model formulae may contain interactions, but they are
  ignored in the graph.} 
\seealso{\code{\link{fitDag}}, \code{\link{UG}}, \code{\link{topSort}}}
\examples{
## A Markov chain
DAG(y ~ x, x ~ z, z ~ u)

## Another DAG
DAG(y ~ x + z + u, x ~ u, z ~ u)

## A DAG with an isolated node
DAG(v ~ v, y ~ x + z, z ~ w + u)

## There can be repetitions
DAG(y ~ x + u + v, y ~ z, u ~ v + z)

## Interactions are ignored
DAG(y ~ x*z + z*v, x ~ z)

## A cyclic graph returns an error!
\dontrun{DAG(y ~ x, x ~ z, z ~ y)}

## Forcing a cyclic graph!
DAG(y ~ x, x ~ z, z ~ y, test=FALSE)

## The order can be changed
DAG(y ~ z, y ~ x + u + v,  u ~ v + z)

## If you want to order the nodes (top sort of the DAG)
DAG(y ~ z, y ~ x + u + v,  u ~ v + z, order=TRUE)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{In}
\alias{In}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Indicator matrix}
\description{
  Finds the indicator matrix of the zeros of a matrix.
}
\usage{
In(A)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{A}{a matrix.}
}
\details{
The indicator matrix is a matrix of zeros and ones which has a zero
element iff the corresponding element of \code{A} is (exactly) zero. 
}
\value{a matrix of the same dimensions as \code{A}.
}
\references{
  Wermuth, N. \& Cox, D.R. (2003). Joint response graphs
  and separation induced by triangular systems. Submitted and available at
  \url{http://psystat.sowi.uni-mainz.de}.
}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{DAG}}, \code{\link{inducedCovGraph}}, \code{\link{inducedConGraph}}}
\examples{
## A simple way to find the overall induced coincentration graph
## The DAG on p. 198 of Cox & Wermuth (1996)
A <- DAG(y1 ~ y2 + y3, y3 ~ y5, y4 ~ y5)
In(crossprod(A))
}
\keyword{array}
\keyword{algebra}
\keyword{graphs}
\keyword{multivariate}
\eof
\name{InducedGraphs}
\alias{inducedCovGraph}
\alias{inducedConGraph}
\alias{InducedGraphs}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Graphs induced by marginalization or conditioning}
\description{
  Functions to find the induced covariance or concentration graphs after
  conditioning on a set of variables and marginalising over another set. 
}
\usage{
inducedCovGraph(A, sel = 1:nrow(A), cond = NULL)
inducedConGraph(A, sel = 1:nrow(A), cond = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{A}{a Boolean matrix with dimnames representing a directed
    acyclic graph. The function does not check if this is the case. See
  the function \code{is.acyclic}}.
  \item{sel}{a vector representing a subset of selected of variables. 
  The vector should be a character vector of the
    names of the variables matching the names of the nodes in
    \code{rownames(A)}. It can be also a numeric vector of indices.
    By default \code{sel} is the set of the nodes of the DAG and
    the \code{cond} is the empty set. 
  }
  \item{cond}{a set of nodes representing the variables on which you
    want to condition. The mode of this vector must match the mode
    of \code{sel}. \code{cond} must be disjoint from \code{sel} and
    their union must be a subset of the set of nodes. The set difference
    between the set of nodes and the union of \code{sel} and
    \code{cond} contains the variables over which we marginalize.
    \code{cond} may be the null vector, meaning that you want
    to condition on the empty set.
  }
}
\details{Given a directed acyclic graph representing a set of
  conditional independencies it is possible to obtain other
  graphs of conditional independence implied after marginalizing
  over and conditionig on sets of nodes. Two such graphs are
  the covariance graph and the concentration graph
  (cfr. Cox \& Wermuth, 1996, 2003).
}
\value{
  \code{inducedCovGraph} returns the edge matrix of the covariance
  graph of the variables in set \code{sel} given the variables
  in set \code{cond}, implied by the original directed acyclic graph
  with edge matrix \code{A}. 
  
  \code{inducedConGraph} returns the edge matrix of the concentration
  graph of the variables in set \code{sel} given the variables
  in set \code{cond}, implied by the original directed acyclic graph
  with edge matrix \code{A}.
  
  If \code{sel} is \code{NULL} the functions return \code{NULL} or the null matrix.
  If \code{cond} is \code{NULL}, the conditioning set is empty and the
  functions return the marginal induced covariance or concentration
  matrices of the selected variables.
}
\note{
  If you do not specify \code{sel} you cannot specify a non \code{NULL}
  value of \code{cond}.
  }
  \references{
    Cox, D. R. \& Wermuth, N. (1996). \emph{Multivariate
      dependencies}. London: Chapman \& Hall.
    
    Wermuth, N. \& Cox, D.R. (2003). Joint response graphs
    and separation induced by triangular systems. Submitted and available at
    \url{http://psystat.sowi.uni-mainz.de}.
  }
\author{Giovanni M. Marchetti}
\seealso{\code{\link{DAG}}, \code{\link{is.acyclic}}}
\examples{
## Define a DAG
dag <- DAG(a ~ x, c ~ b+d, d~ x)
dag
## Induced covariance graph of a, b, d given the empty set.
inducedCovGraph(dag, sel=c("a", "b", "d"), cond=NULL)

## Induced concentration graph of a, b, c given x
inducedConGraph(dag, sel=c("a", "b", "c"), cond="x")

## Overall covariance graph
inducedCovGraph(dag)

## Overall concentration graph
inducedConGraph(dag)

## Induced covariance graph of x, b, d given c, x.
inducedCovGraph(dag, sel=c("a", "b", "d"), cond=c("c", "x"))

## Induced concentration graph of a, x, c given d, b.
inducedConGraph(dag, sel=c("a", "x", "c"), cond=c("d", "b"))

## The variables may be specified by numeric vectors (both)
inducedConGraph(dag, sel=c(1,4,2), cond=c(3, 5))

## The DAG on p. 198 of Cox & Wermuth (1996)
dag <- DAG(y1~ y2 + y3, y3 ~ y5, y4 ~ y5)

## Cfr. figure 8.7 p. 203 in Cox & Wermuth (1996)
inducedCovGraph(dag, sel=c("y2", "y3", "y4", "y5"), cond="y1")
inducedCovGraph(dag, sel=c("y1", "y2", "y4", "y5"), cond="y3")
inducedCovGraph(dag, sel=c("y1", "y2", "y3", "y4"), cond="y5")

## Cfr. figure 8.8 p. 203 in Cox & Wermuth (1996)
inducedConGraph(dag, sel=c("y2", "y3", "y4", "y5"), cond="y1")
inducedConGraph(dag, sel=c("y1", "y2", "y4", "y5"), cond="y3")
inducedConGraph(dag, sel=c("y1", "y2", "y3", "y4"), cond="y5")

## Cfr. figure 8.9 p. 204 in Cox & Wermuth (1996)
inducedCovGraph(dag, sel=c("y2", "y3", "y4", "y5"), cond=NULL)
inducedCovGraph(dag, sel=c("y1", "y2", "y4", "y5"), cond=NULL)
inducedCovGraph(dag, sel=c("y1", "y2", "y3", "y4"), cond=NULL)

## Cfr. figure 8.10 p. 204 in Cox & Wermuth (1996)
inducedConGraph(dag, sel=c("y2", "y3", "y4", "y5"), cond=NULL)
inducedConGraph(dag, sel=c("y1", "y2", "y4", "y5"), cond=NULL)
inducedConGraph(dag, sel=c("y1", "y2", "y3", "y4"), cond=NULL)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{Simple Graph Operations}
\alias{bd}
\alias{ch}
\alias{pa}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Simple graph operations}
\description{
Finds the boundary, children, parents of a subset of nodes of a graph.
}
\usage{
bd(nn, A)
ch(nn, A)
pa(nn, A)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nn}{a vector of nodes. It may either a numeric vector, or a
    character vector. If it is character vector must be a subset
    of the \code{rownames} of the edge matrix.}
  \item{A}{a square matrix specifying the edge matrix of the graph}
}
\details{For the definitions of these operators see Lauritzen (1996).
}
\value{
  A vector, specifying the boundary or the children or the parents of nodes  \code{nn}
  in the graph.  This is a numeric or a character vector depending
  on the mode of \code{nn}.
}
\references{Lauritzen, S. (1996). \emph{Graphical models}. Oxford:
Clarendon Press.}

\author{Giovanni M. Marchetti}
\seealso{\code{\link{UG}}, \code{\link{DAG}}}
\examples{
## find boundary of a subset of nodes of a DAG
G <- DAG(y ~ x+b+a, b~a, x~a)
bd("b", G)
bd(c("b", "x"), G)
bd(2, G)
bd(c(2,3), G)
## find boundary of a subset of nodes of an UG
G <- UG(~ y*x*z + z*h*v)
bd("z", G)
bd(c("y", "x"), G)
bd(5, G)
bd(c(2,5), G)
## children of a subset of nodes of a DAG
G <- DAG(y ~ x+b+a, b~a, x~a)
ch("b", G)
ch(c("b", "x"), G)
ch(2, G)
ch(c(2,3), G)
## parents of a subset of nodes of a DAG
pa("b", G)
pa(c("b", "x"), G)
pa(2, G)
pa(c(2,3), G)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{UG}
\alias{UG}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Defining an undirected graph (UG)}
\description{
  A simple way to define an undirected graph by means of a single
  model formula.
}
\usage{
UG(f)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{f}{a single model formula without response}
}
\details{
  The undirected graph \eqn{G = (V, E)} is defined by a set of nodes
  \eqn{V} and a set of pairs \eqn{E}. The set of pairs is defined by
  the set of interactions in the formula. Interactions 
  define complete subgraphs (not necessarily maximal) of the UG.
  The best way is to specify interactions that match the cliques
  of the undirected graph. This is the standard way to define
  graphical models for contingency tables. Remember that some
  hierarchical models are not graphical, but they imply the same graph.

  The function returns the edge matrix of the graph, i.e. 
  a square Boolean matrix of order equal to the number of nodes of the
  graph and a one in position \eqn{(i,j)} if there is an arrow from
  \eqn{j} to \eqn{i} and zero otherwise. By default this matrix
  has ones along the main diagonal. For UGs this matrix is symmetric.
  The dimnames of the edge matrix are the nodes of the UG.
}
\value{
the edge matrix of the undirected graph.
}
\references{Lauritzen, S. (1996). \emph{Graphical models}. Oxford:
Clarendon Press.}
\author{Giovanni M. Marchetti}
\note{} 
\seealso{\code{\link{fitUg}}, \code{\link{DAG}}}
\examples{
## X independent of Y given Z
UG(~ X*Z + Y*Z)

# The saturated model
UG(~ X*Y*Z)

## The model without three-way interactions has the same graph
UG(~ X*Y + Y*Z + Z*X)
UG(~ (X + Y + Z)^2)

## Butterfly model defined from the cliques
UG(~ mec*vec*alg + alg*ana*sta)

## Some isolated nodes
UG(~x*y*z + a + b) 
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{basiSet}
\alias{basiSet}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Basis set of a DAG}
\description{
  Finds a basis set for the conditional independencies
  implied by a directed acyclic graph, that is a minimal set of
  independencies that imply all the other ones.
}
\usage{
basiSet(A)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{A}{a square Boolean matrix with dimnames representing the edge
    matrix of a DAG.}
}
\details{
  Given a DAG and a pair of non adjacent nodes
  \eqn{(i,j)} such that \eqn{j} has higher causal order than \eqn{i},
  the set of independency statements \eqn{i} independent of
  \eqn{j} given the union of the parents of both \eqn{i} and \eqn{j}
  is a basis set (cfr. Shipley, 2000). This basis set has the property
  to lead to independent test statistics.
}
\value{
  a list of vectors representing several conditional independence
  statements. Each vector contains the names of two non adjacent
  nodes plus the names of the nodes in the conditioning set (which
  may be empty).
}
\references{Shipley, B. (2000). A new inferential test
for path models based on directed acyclic graphs. \emph{Structural
  Equation Modeling}, 7(2), 206--218.}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{shipley.test}}, \code{\link{dSep}}, \code{\link{DAG}}}
\examples{
## See Shipley (2000), Figure 2, p. 213
A <- DAG(x5~ x3+x4, x3~ x2, x4~x2, x2~ x1)
basiSet(A)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{bfs}
\alias{bfs}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Breadth first search}
\description{Breadth-first search of a connected undirected graph.
}
\usage{
bfs(gmat, v = 1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a symmetric matrix with dimnames specifying the edge
    matrix of the undirected graph}
  \item{v}{an integer, indicating the starting node of the
    search. Defaults to the first node.
  }
}
\details{
  Breadth-first search is a systematic method for exploring a graph.
  The algorithms is taken from Aho, Hopcroft \& Ullman (1983).
}
\value{
  \item{tree}{the edge matrix of the resulting spanning tree}
  \item{branches}{a matrix with two columns, giving the indices of the
    branches of the spanning tree}
  \item{chords}{a matrix with two columns, giving the indices of the
    chords of the spanning tree}
}
\references{
Aho, A.V., Hopcrtoft, J.E. \& Ullman, J.D. (1983). \emph{Data structures
  and algorithms.} Reading: Addison-Wesley.

  Thulasiraman, K. \& Swamy, M.N.S. (1992). \emph{Graphs: theory and
    algorithms}. New York: Wiley.
}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{UG}},\code{\link{findPath}}, \code{\link{cycleMatrix}}}
\examples{
## Finding a spanning tree of the butterfly graph
bfs(UG(~ a*b*o + o*u*j))
## Starting from another node
bfs(UG(~ a*b*o + o*u*j), v=3)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{checkIdent}
\alias{checkIdent}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Identifiability of a model with one latent variable}
\description{
  Checks  four sufficient conditions for identifiability of a Gaussian DAG model
with one latent variable.
}
\usage{
checkIdent(gmat, hidden)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix with dimnames, the edge matrix of
    a DAG.}
  \item{hidden}{an integer representing the latent variables among the
    nodes, or the name of the node.}
}
\details{
  Stanghellini and Wermuth (2003) give some
  sufficient conditions for checking if a Gaussian
  model that factorizes according to a DAG is identified
  when there is one hidden node over which we marginalize.
  Specifically, the function checks the conditions of
  Theorem 1, (i) and (ii) and of Theorem 2 (i) and (ii).
}
\value{
  a vector of logical values, indicating if each of the  four 
  conditions of theorems 1 and 2 in Stanghellini \& Wermuth are
  \code{TRUE} or \code{FALSE}.
}
\references{
  Stanghellini, E. \& Wermuth, N. (2003). On the
  identification of directed acyclic graph models with one
  hidden variable. Submitted and available at
  \url{http://psystat.sowi.uni-mainz.de}.
}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{is.Gident}},  \code{\link{InducedGraphs}}}
\examples{
## See DAG in Figure 4 (a) in Stanghellini & Wermuth (2003)
d <- DAG(y1 ~ y3, y2 ~ y3 + y5, y3 ~ y4 + y5, y4 ~ y6)
checkIdent(d, "y3")  # Identifiable
checkIdent(d, "y4")  # Not identifiable

## See DAG in Figure 5 (a) in Stanghellini & Wermuth (2003)
d <- DAG(y1 ~ y5+y4, y2 ~ y5+y4, y3 ~ y5+y4)
checkIdent(d, "y4")  # Identifiable
checkIdent(d, "y5")  # Identifiable

## A simple function to check identifiability for each node
"is.ident" <-
function(gmat, compact=TRUE){
### Check suff. conditions marginalising on each node of a DAG. 
    a <- rownames(gmat)
    for(i in a) {
        b <- checkIdent(gmat, hidden=i)
      if(compact)
        b <- any(b)
      
      cat("Node: ", i, "\n")
      print(b)
    }
  }

}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}

\eof
\name{cliques}
\alias{cliques}
\title{Cliques of an undirected graph}
\description{
Finds the cliques of an undirected graph.
}
\usage{
cliques(gmat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix with dimnames, representing the
  edge matrix of an undirected graph.}
}
\details{
  The cliques of a graph are the subsets of nodes which induce a
  maximally complete subgraph. The algorithm is NP-complete in general.
}
\value{
  a list of vectors of integers representing  subsets of the set of nodes. 
}
\references{
  Lauritzen, S. (1996). \emph{Graphical models}. Oxford:
  Clarendon Press.}
\author{The code was written by somebody at  Lancaster University.}
\seealso{\code{\link{UG}},\code{\link{bd}}}
\examples{
## Two cliques
u <- UG(~ a*b*c + c*d*e*g)
u
cliques(u)

## the same (note that the formula is not of a graphical model)
u <- UG(~ a*b+b*c+c*a + c*d*e*g)
u
cliques(u)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}

\eof
\name{clos}
\alias{clos}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Graph operations}
\description{
Transforms an edge matrix of a graph.
}
\usage{
clos(M)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{M}{a square matrix with dimnames representing the edge matrix of
  an undirected graph}
}
\details{
  The function \code{clos} is not intended to be used directly
  from the user. It is used by functions that compute the induced
  covariance or concentration graphs. The function \code{clos}
  is explained by Cox \& Wermuth (2003).
}
\value{
  a Boolean matrix of the same dimensions of \code{M}.}
\references{
  Wermuth, N. \& Cox, D.R. (2003). Joint response graphs
  and separation induced by triangular systems. Submitted and available at
  \url{http://psystat.sowi.uni-mainz.de}.
  }
\author{Giovanni M. Marchetti} 
\seealso{\code{\link{DAG}}, \code{\link{inducedCovGraph}}, \code{\link{inducedConGraph}}}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}

\eof
\name{cmpGraph}
\alias{cmpGraph}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{The complementary graph}
\description{
Finds the complementary graph of an undirected graph.
}
\usage{
cmpGraph(gmat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{the edge matrix of an undirected graph}
}
\details{
  The complementary graph of an UG is the graph that has the same set
  of nodes and an undirected edge connecting \eqn{i} and \eqn{j}
  whenever there is not an \eqn{(i,j)} edge in the original UG.
}
\value{
the edge matrix of the complementary graph.
}
\references{Lauritzen, S. (1996). \emph{Graphical models}. Oxford:
Clarendon Press.}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{UG}}, \code{\link{DAG}}}
\examples{
## A chordless four-cycle
four <- UG(~ a*b + b*d + d*e + e*a)
print(four)
cmpGraph(four)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{conComp}
\alias{conComp}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Connectivity components}
\description{
Finds the connectivity components of a graph.
}
\usage{
conComp(gmat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix with dimnames, the edge matrix of
    a DAG or a UG.}
}
\value{
  an integer vector representing a partition of the set of nodes.
}
\references{Lauritzen, S. (1996). \emph{Graphical models}. Oxford:
Clarendon Press.}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{UG}}, \code{\link{anGraph}}}
\examples{
## three connected components
conComp(UG(~a*c+c*d+e+g*o*u))
## a connected graph
conComp(UG(~ a*b+b*c+c*d+d*a))
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{correlations}
\alias{correlations}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Marginal and partial correlations}
\description{
  Computes a correlation matrix with ones along the
  diagonal, marginal correlations in the lower triangle and
  partial correlations given all remaining variables in the
  upper triangle.
}
\usage{
correlations(x)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{a square symmetric matrix, a covariance matrix, or a
  data.frame for n observations and p variables.}
}
\value{
  a square correlation matrix with marginal correlations (lower
  triangle) and partial correlations (upper triangle).
}
\references{Cox, D. R. \& Wermuth, N. (1996). \emph{Multivariate
  dependencies}. London: Chapman \& Hall.}
\author{Giovanni M. Marchetti} 
\seealso{\code{\link{parcor}},  \code{\link{cor}}}
\examples{
## See Table 6.1 in Wermuth & Cox (1996)
data(glucose)
correlations(glucose)
}
\keyword{array}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}

\eof
\name{cycleMatrix}
\alias{cycleMatrix}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Fundamental cycles}
\description{
Finds the matrix of fundamental cycles of a connected undirected graph. 
}
\usage{
cycleMatrix(gmat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a symmetric matrix with dimnames denoting the edge matrix
    of the undirected graph. The graph must be connected, otherwise
    the function returns an error message.}
}
\details{
  All the cycles in an UG can be obtained from combination (ring sum)
  of the set of fundamental cycles. The matrix of fundamental cycles
  is a Boolean matrix having as rows the fundamental cycles and as
  columns the edges of the graph. If an entry is one then the edge
  associated to that column belongs to the cycle associated to the row.
}
\value{
  a Boolean matrix of the fundamental cycles of the undirected graph.
  If   there is no cycle the function returns \code{NULL}.}
\references{
  Thulasiraman, K. \& Swamy, M.N.S. (1992). \emph{Graphs: theory and
    algorithms}. New York: Wiley.
}
\author{Giovanni M. Marchetti}
\note{This function is used by \code{is.Gident}. The row sum of the
  matrix gives the length of the cycles.}
\seealso{\code{\link{UG}}, \code{\link{findPath}},
  \code{\link{fundCycles}}, \code{\link{is.Gident}}, \code{\link{bfs}}}
  \examples{
## Three cycles
cycleMatrix(UG(~a*b*d+d*e+e*a*f))
## No cycle
 cycleMatrix(UG(~a*b))
## two cycles: the first is even and the second is odd
cm <- cycleMatrix(UG(~a*b+b*c+c*d+d*a+a*u*v))
apply(cm, 1, sum)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{dSep}
\alias{dSep}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{d-separation}
\description{
Determines if in a directed acyclic graph two set of nodes a d-separated
by a third set of nodes.
}
\usage{
dSep(gmat, first, second, cond)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a Boolean matrix with dimnames, representing a directed
    acyclic graph. The function does not check if this is the case. See
    the function \code{is.acyclic}. }
  \item{first}{a vector representing  a subset of nodes of the DAG.
    The vector should be a character vector of the
    names of the variables matching the names of the nodes in
    \code{rownames(A)}. It can be also a numeric vector of indices. }
  \item{second}{a vector representing  another subset of nodes of the DAG.
    The set \code{second} must be disjoint from \code{first}.
    The mode of \code{second} must match the mode of \code{first}.}
  \item{cond}{a vector representing a conditioning subset of nodes.
    The set \code{cond} must be disjoint from the other two sets and
    must share the same mode.
  }
}
\details{
 d-separation is a fundamental concept discovered by Pearl (1988). 
}
\value{
  a logical value. \code{TRUE} if \code{first} and \code{second} are d-separated
  by \code{cond}. 
}
\references{
  Pearl, J. (1988). \emph{Probabilistic reasoning in intelligent
    systems.} San Mateo: Morgan Kaufmann.
  
  Lauritzen, S. (1996). \emph{Graphical models}. Oxford:
  Clarendon Press.
}
\author{Giovanni M. Marchetti} 
\seealso{\code{\link{DAG}}, \code{\link{shipley.test}},
  \code{\link{inducedCovGraph}}}
\examples{
## Conditioning on a transition node
dSep(DAG(y ~ x, x ~ z), first="y", second="z", cond = "x")
## Conditioning on a collision node (collider)
dSep(DAG(y ~ x, y ~ z), first="x", second="z", cond = "y")
## Conditioning on a source node
dSep(DAG(y ~ x, z ~ x), first="y", second="z", cond = "x")
## Marginal independence
dSep(DAG(y ~ x, y ~ z), first="x", second="z", cond = NULL)
## The DAG defined on p.~47 of Lauritzen (1996)
dag <- DAG(g ~ x, h ~ x+f, f ~ b, x ~ l+d, d ~ c, c ~ a, l ~ y, y ~ b)
dSep(dag, first="a", second="b", cond=c("x", "y"))
dSep(dag, first="a", second=c("b", "d"), cond=c("x", "y"))
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{edges}
\alias{edges}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Edges of a graph}
\description{
  Finds the set of edges of a graph. That is
  the set of undirected edges if the graph is undirected
  and the set of arrows if the graph is directed.
}
\usage{
edges(gmat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix, with dimnames, and edge matrix of
    a graph.}
}
\value{
  a matrix with two columns. Each row of the matrix is a pair of
  indices indicating an edge of the graph. If the graph is undirected,
  then only one of the pairs \eqn{(i,j), (j,i)} is reported.
}
\author{Giovanni M. Marchetti} 
\seealso{\code{\link{cycleMatrix}}}
\examples{
## A UG graph
edges(UG(~ y*v*k +v*k*d+y*d))

## A DAG
edges(DAG(u~h+o+p, h~o, o~p))
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}

\eof
\name{findPath}
\alias{findPath}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Finding paths}
\description{
Finds one path between two nodes of a graph.
}
\usage{
findPath(gmat, st, en, path = c())
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix with dimnames, the edge matrix of a graph.}
  \item{st}{an integer, the starting node.}
  \item{en}{an integer, the ending node.}
  \item{path}{a vector of integers, used in recursive calls. At the
    beginning is \code{NULL}. It should not be modified by the user.}
}
\value{
  a vector of integers, the sequence of nodes of a path,
  starting from \code{st} to \code{en}. In some graphs
  (spanning trees) there is only one path between two nodes.
}
\note{
  This function is not intended to be directly called by the user.
}
\references{Python Softftware Foundation (2003). Python Patterns ---
  Implementing Graphs. \url{http://www.python.org/docs/essays/graphs.html}.}
\author{Giovanni M. Marchetti, translating the original python code (see
references).}
\seealso{\code{\link{fundCycles}}}
\examples{
## A (single) path on a spanning tree
findPath(bfs(UG(~ a*b*c + b*d + d*e+ e*c))$tree, st=1, en=5)
}
\keyword{graphs}
\eof
\name{fitDag}
\alias{fitDag}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Gaussian Markov models specified by a DAG}
\description{
  Fits linear recursive regressions with independent residuals specified
  by a DAG.
}
\usage{
fitDag(gmat, Syy, n)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix representing
    the edge matrix of the DAG}
  \item{Syy}{a symmetric positive definite matrix, the
    sample covariance matrix}
  \item{n}{an integer >0, the sample size}
}
\details{
  \code{fitDag} checks if the order of the nodes in edge matrix
  is the same of \code{Syy} and if not it reorders the edge matrix
  to match the order of the variables in \code{Syy}. The nodes
  of the edge matrix may form a subset of the variables in \code{Syy}.
}
\value{
  \item{A}{the matrix of the fitted regression coefficients. Each
    coefficient has the sign reversed.}
  \item{B}{the inverse of matrix B.}
  \item{Delta}{a vector containing the partial covariances of each
    variable given the parents.}
  \item{Shat}{the fitted covariance matrix.}
  \item{Khat}{the fitted concentration matrix.}
  \item{dev}{the `deviance' (\eqn{-2 \log L}) of the model.}
  \item{df}{the degrees of freedom.}
}
\references{Cox, D. R. \& Wermuth, N. (1996). \emph{Multivariate
  dependencies}. London: Chapman \& Hall.}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{DAG}}, \code{\link{swp}}.}
\examples{
dag <- DAG(y ~ x+u, x ~ z, z ~ u)
"V" <- structure(c(2.93, -1.7, 0.76, -0.06,
                   -1.7, 1.64, -0.78, 0.1,
                    0.76, -0.78, 1.66, -0.78,
                    -0.06, 0.1, -0.78, 0.81), .Dim = c(4,4),
         .Dimnames = list(c("y", "x", "z", "u"), c("y", "x", "z", "u")))
fitDag(dag, V, 200)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{fitDagLatent}
\alias{fitDagLatent}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Gaussian DAG model with one latent variable}
\description{
Fits by maximum likelihood a Gaussian DAG model where one of the nodes
of the graph is latent and it is marginalised over.
}
\usage{
fitDagLatent(gmat, Syy, n, latent, norm = 1, seed = 144, maxit = 9000, tol = 1e-06, pri = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix with dimnames, the edge matrix of
    the DAG.}
  \item{Syy}{a symmetric positive definite matrix, with dimnames, the sample covariance
  matrix. The order of the rows needs not match the order of the nodes
  of the graph. However, the set of nodes of the graph must be a subset
  of the set of the names of the variables in \code{Syy}.}
  \item{n}{a positive integer, the sample size.}
  \item{latent}{the name of the hidden node.}
  \item{norm}{an integer, the kind of normalization of the latent
    variable.
    If \code{norm=1}, the latent is scaled to have unit variance. If
    \code{norm=2}, the latent is scaled to have unit partial variance
    given its parents.}
  \item{seed}{an integer, used by \code{set.seed} to specify a random
  starting point of the EM algorithm.}
\item{maxit}{an integer denoting the maximum number of iterations
  allowed for the EM algorithm. If the convergence criterion is not
  satisfied within maxit iterations the algorithms stops and a warning
  message is returned.}
\item{tol}{a small real value, denoting the tolerance used in testing convergence.}
\item{pri}{logical, if \code{pri=TRUE} then the value of the deviance at
  each iteration is printed.}
}
\details{
The EM algorithm used is due to Kiivery (1987).  
}
\value{
  \item{Shat}{
    a symmetric matrix, the fitted covariance matrix of all the variables
    including the latent one. 
  }
  \item{Khat}{
    a symmetric matrix, the fitted concentration matrix,
    i.e. the inverse of \code{Shat}.
  }
  \item{A}{
    a square matrix with ones along the diagonal,
    resulting from the triangular decomposition
    of \code{Shat}. The non diagonal entries are partial regression
    coefficients, with sign changed, attached to the edges of the fitted
    DAG.
  }
  \item{Delta}{
    a numeric vector resulting from triangular decomposition of
    \code{Shat}. The values are the partial variances.
  }
  \item{dev}{
    the `deviance' (\eqn{-2 \log L}) of the model.
    }
    \item{it}{a positive integer, the number of EM algorithm iterations at convergence.}
  \item{df}{a positive integer, degrees of freedom of the model.}
}
\references{
  Kiiveri,H. T. (1987). An incomplete data approach to the
  analysis of covariance structures. \emph{Psychometrika}, 52, 4,
  539--554.

  J\"oreskog, K.G. \& Goldberger, A.S. (1975). Estimation of a model
  with multiple indicators and multiple causes of a single latent
  variable.
  \emph{Journal of the American Statistical Association}, 10, 631--639.
}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{fitDag}}, \code{\link{checkIdent}}}
\examples{
## data from Joreskog and Goldberger (1975)
V <- matrix(c(1,     0.36,   0.21,  0.10,  0.156, 0.158,
              0.36,  1,      0.265, 0.284, 0.192, 0.324,
              0.210, 0.265,  1,     0.176, 0.136, 0.226,
              0.1,   0.284,  0.176, 1,     0.304, 0.305, 
              0.156, 0.192,  0.136, 0.304, 1,     0.344,
              0.158, 0.324,  0.226, 0.305, 0.344, 1),     6,6)
nod <- c("y1", "y2", "y3", "x1", "x2", "x3")
dimnames(V) <- list(nod,nod)
dag <- DAG(y1 ~ z, y2 ~ z, y3 ~ z, z ~ x1 + x2 + x3, x1~x2+x3, x2~x3) 
fitDagLatent(dag, V, n=530, latent="z", seed=4564)
fitDagLatent(dag, V, n=530, latent="z", norm=2, seed=145)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{fitUg}
\alias{fitUg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Gaussian Markov models specified by an UG}
\description{
  Fits a concentration graph (a covariance selection model) or
  a covariance graph to a sample covariance matrix, assuming
  a Gaussian model.
}
\usage{
fitUg(gmat, V, n, cov = FALSE, pri = FALSE, tol = 1e-06)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix representing
    the edge matrix of the DAG}
  \item{V}{a symmetric positive definite matrix, the
    sample covariance matrix}
  \item{n}{an integer >0, the sample size}
  \item{cov}{a logical value. If \code{cov=TRUE}, then
    it is fitted a covariance graph. Else it is fitted
    a concentration graph.}
  \item{pri}{a logical value. If \code{TRUE} a the value
    of the criterion at each iteration is printed.}
  \item{tol}{a positive number indicating the tolerance
    used in convergence tests.
  }
}

\details{
  Algorithms for fitting Gaussian graphical models specified
  by undirected graphs are discussed in Speed \& Kiiveri (1986).
  This function is based on the iterative proportional
  fitting algorithm described on p. 184 of Whittaker (1990). 
}
\value{
  \item{type}{a character string indicating the type of the graph,
  concentration or covariance.}
\item{cliques}{a list of numeric vectors indicating the cliques
of the UG.}
\item{Vhat}{the fitted covariance matrix.}
\item{Rhat}{the fitted correlation matrix.}
\item{Phat}{the fitted partial correlation matrix.}
\item{dev}{the `deviance' (\eqn{-2 \log L}) of the model.}
\item{df}{the degrees of freedom}
}
\references{
    Cox, D. R. \& Wermuth, N. (1996). \emph{Multivariate
      dependencies}. London: Chapman \& Hall.
      
  Speed, T.P. \& Kiiveri, H (1986). Gaussian
  Markov distributions over finite graphs. \emph{Annals of
    Statistics}, 14, 138--150.
  
  Whittaker, J. (1990). \emph{Graphical models in applied
    multivariate statistics}. Chichester: Wiley.
}
\author{Giovanni M. Marchetti} 
\seealso{\code{\link{UG}}, \code{\link{fitDag}},
  \code{\link{cliques}}, \code{marks}}
\examples{
## A model for the sample covariance matrix of the
## mathematics marks (cfr. Whittaker, 1990)
data(marks)
V <- cov(marks) * 87 / 88
## A butterfly concentration graph
fitUg(UG(~ mec*vec*alg + alg*ana*sta),V , n=88)

## Correlations among four strategies to cope with stress for 
## 72 students. Cfr. Cox & Wermuth (1996), p. 73.
##  Y = cognitive avoidance
##  X = vigilance
##  V = blunting
##  U = monitoring

R <- matrix(c(
   1.00, -0.20,  0.46,  0.01,
  -0.20,  1.00,  0.00,  0.47,
   0.46,  0.00,  1.00, -0.15,
   0.01,  0.47, -0.15,  1.00), 4, 4)
nam <- c("Y", "X", "V", "U") 
dimnames(R) <- list(nam, nam)

## A chordless 4-cycle covariance graph

gr <- UG(~ Y*X + X*U + U*V + V*Y)
fitUg(gr, R, 72, cov=TRUE)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{fundCycles}
\alias{fundCycles}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Fundamental cycles}
\description{
Finds the list of fundamental cycles of a connected undirected graph.
}
\usage{
fundCycles(gmat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a symmetric matrix with dimnames denoting the edge matrix
    of the undirected graph. The graph must be connected, otherwise
    the function returns an error message.}
}
\details{
  All the cycles in an UG can be obtained from combination (ring sum)
  of the set of fundamental cycles. 
}
\value{
  a list of matrices with two columns. Every component of the list
  is associated to a cycle. The cycle is described by a
  \eqn{k \times 2} matrix whose rows are the edges of the cycle. If
  there is no cycle the function returns \code{NULL}.
}
\references{
  Thulasiraman, K. \& Swamy, M.N.S. (1992). \emph{Graphs: theory and
    algorithms}. New York: Wiley.
}
\author{Giovanni M. Marchetti}
\note{This function is used by \code{cycleMatrix} and \code{is.Gident}.}
\seealso{\code{\link{UG}},\code{\link{findPath}},
  \code{\link{cycleMatrix}}, \code{\link{is.Gident}},\code{\link{bfs}}}
  \examples{
## One cycle
cycleMatrix(UG(~a*b*d + d*e + e*a*f))
## Three fundamental cycles
fundCycles(UG(~a*b*d + d*e + e*a*f))
## No cycles
cycleMatrix(UG(~a*b + b*d + d*e))
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}

\eof
\name{glucose}
\alias{glucose}
\docType{data}
\title{Glucose control}
\description{
Data on glucose control of diabetes patients.  
}
\usage{data(glucose)}
\format{
  A data frame with 68 observations on the following 8 variables.
  \describe{
    \item{Y}{a numeric vector, Glucose control (glycosylated
    haemoglobin), values up to about 7 or 8 indicate good glucose control.}
    \item{X}{a numeric vector, a score for knowdledge about the illness.} 
    \item{Z}{a numeric vector, a score for fatalistic externality (mere
      chance determines what occurs).}
    \item{U}{a numeric vector, a score for social externaliy (powerful
      others are responsible).}
    \item{V}{a numeric vector, a score for internality (the patient is
    him or herself responsible).}
    \item{W}{a numeric vector, duration of the illness in years.}
    \item{A}{a factor, level of education, with levels \code{1}: at least
      13 years of formal schooling, \code{2}: less then 13 years.}
    \item{B}{a factor, gender with levels \code{1}: females, \code{2}: males.}
  }
}
\details{
  Data on 68 patients with fewer than 25 years of diabetes. They were
  collected at the University of Mainz to identify psychological
  and socio-economic variables possibly important for glucose control,
  when patients choose the appropriate dose of treatment depending
  on the level of blood glucose measured several times per day.  

  The variable   of primary interest is \code{Y}, glucose control, measured
  by  glycosylated haemoglobin. \code{X}, knowdledge about the illness,
  is a response of secondary interest. Variables \code{Z}, \code{U} and
  \code{V} measure patients' type of attribution, called fatalistic
  externality, social externality and internality. These are intermediate
  variables. Background   variables are \code{W}, the duration of the
  illness, \code{A} the duration of formal schooling and \code{A},
  gender. These are intrinsic variables.
}
\source{
Cox & Wermuth (1996), p. 229.
}
\references{Cox, D. R. \& Wermuth, N. (1996). \emph{Multivariate
  dependencies}. London: Chapman \& Hall.}
\examples{
data(glucose)
## See Cox & Wermuth (1996), Figure 6.3 p. 140 
coplot(Y ~ W | A, data=glucose) 
}
\keyword{datasets}
\eof
\name{is.Gident}
\alias{is.Gident}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{G-identifiability of an UG}
\description{
Tests if an undirected graph is G-identifiable. 
}
\usage{
is.Gident(gmat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a symmetric matrix with dimnames representing the edge
    matrix of an undirected graph}
}
\details{
  An undirected graph is G-identifiable if every connected component of
  the complementary graph contains an odd cycle (cfr. Stanghellini and
  Wermuth, 2003). The algorithm used follows suggestions by
  Tarantola and Vicard (2002).
}
\value{
  a logical value, \code{TRUE} if the graph is G-identifiable and
  \code{FALSE} if it is not.
}
\references{Stanghellini, E. \& Wermuth, N. (2003). On the
  identification of directed acyclic graph models with one
  hidden variable. Submitted and available at
  \url{http://psystat.sowi.uni-mainz.de}.
  
  Stanghellini, E. (1997). Identification of a single-factor
  model using graphical Gaussian rules. \emph{Biometrika}, 87, 199--205.

  Tarantola, C. \& Vicard, P. (2002). Spanning trees and identifiability
  of a single-factor model. \emph{Statistical Methods \& Applications},
  11, 139--152.
  
  Vicard, P. (2000). On the identification of a single-factor model
  with correlated residuals. \emph{Biometrika}, 87, 199--205.
}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{UG}}, \code{\link{cycleMatrix}}}
\examples{
## A not G-identifiable UG
G1 <- UG(~ a*b + u*v)
is.Gident(G1)
## G-identifiable UG
G2 <- UG(~ a + b + u*v)
is.Gident(G2)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}

\eof
\name{is.acyclic}
\alias{is.acyclic}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Graph queries}
\description{
  Checks if a given graph is acyclic.
}
\usage{
is.acyclic(A)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{A}{a square Boolean matrix with dimnames, the edge matrix of a graph.}
}
\value{
  a logical value, \code{TRUE} if the graph is acyclic and \code{FALSE}
  otherwise.
}
\references{
  Aho, A.V., Hopcrtoft, J.E. \& Ullman, J.D. (1983). \emph{Data structures
    and algorithms.} Reading: Addison-Wesley.
}
\author{Giovanni M. Marchetti} 
\seealso{\code{\link{ancGraph}}}
\examples{
## Force a cyclic graph
d <- DAG(y ~ x, x ~ z, z~ y, test=FALSE)
## Test acyclicity
is.acyclic(d)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{marks}
\alias{marks}
\docType{data}
\title{Mathematics marks}
\description{
Examination marks of 88 students in five subjects.
}
\usage{data(marks)}
\format{
  A data frame with 88 observations on the following 5 variables.
  \describe{
    \item{mec}{a numeric vector, mark in Mechanics}
    \item{vec}{a numeric vector, mark in Vectors}
    \item{alg}{a numeric vector, mark in Algebra}
    \item{ana}{a numeric vector, mark in Analysis}
    \item{sta}{a numeric vector, mark in Statistics }
  }
}
\details{
  Mechanics and Vectors were closed book examinations. Algebra, Analysis
  and Statistics were open book examinations.
}
\source{
  Mardia, K.V., Kent, J.T. \& Bibby, (1979). \emph{Multivariate
  analysis}. London: Academic Press.
}
\references{
  Whittaker, J. (1990). \emph{Graphical models in applied
    multivariate statistics}. Chichester: Wiley.
}
\examples{
data(marks)
pairs(marks)
}
\keyword{datasets}
\eof
\name{parcor}
\alias{parcor}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Partial correlations}
\description{
  Finds the matrix of the partial correlations between pairs
  of variables given the rest.
}
\usage{
parcor(Smat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Smat}{a symmetric positive definite matrix, representing a
    covariance matrix.}
}
\details{
  The algorithm computes \eqn{- \sigma^{rs}/(\sigma^{rr}
    \sigma^{ss})^{1/2}} where the \eqn{\sigma}s are concentrations.
  
}
\value{
  A symmetric matrix with ones along the diagonal and in position
  \eqn{(r,s)} the partial correlation between variables \eqn{r}
  and \eqn{s} given all the remaining variables.}
\references{Cox, D. R. \& Wermuth, N. (1996). \emph{Multivariate
    dependencies}. London: Chapman \& Hall.}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{var}}, \code{\link{cor}}, \code{\link{correlations}}}
\examples{
### Partial correlations for the mathematics marks data
data(marks)
S <- var(marks)
parcor(S)
}
\keyword{array}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}

\eof
\name{pcor}
\alias{pcor}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Partial correlation}
\description{
  Computes the partial correlation between two variables given
  a set of other variables.
}
\usage{
pcor(u, S)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{u}{a vector of integers of length > 1. The first two integers are
    the indices of variables the correlation of which must be
    computed. The rest of the vector is the conditioning set.
  }
  \item{S}{a symmetric positive definite matrix, a sample covariance
    matrix.}
}
\value{
  a scalar, the partial correlation matrix between variables \code{u[1]}
  and \code{u[2]} given \code{u[-c(1,2)]}.
}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{cor}},  \code{\link{parcor}}, \code{\link{correlations}}}
\examples{
data(marks)
## The correlation between vec and alg given ana and sta
 pcor(c("vec", "alg", "ana", "sta"), var(marks))
## The same
pcor(c(2,3,4,5), var(marks))
## The correlation between vec and alg given sta
 pcor(c("vec", "alg", "sta"), var(marks))
## The marginal correlation between ana and sta 
pcor(c("ana","sta"), var(marks))
}
\keyword{models}
\keyword{multivariate}
\eof
\name{pcor.test}
\alias{pcor.test}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Test for zero partial association}
\description{
  Test for conditional independence between two variables, given the
  other ones, assuming a multivariate normal distribution.
}
\usage{
pcor.test(r, q, n)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{r}{a partial correlation coefficient, computed by \code{\link{pcor}}.}
  \item{q}{the number of variables in the conditioning set.}
  \item{n}{integer > 0, the sample size.}
}
\value{
  \item{tval}{The Student's t-test statistic.}
  \item{df}{The degrees of freedom}
  \item{pvalue}{The P-value, assuming a two-sided alternative.}
}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{pcor}}, \code{\link{shipley.test}}}
\examples{
## Are 2,3 independent given 1?
data(marks)
pcor.test(pcor(c(2,3,1), var(marks)), 1, n=88)
}
\keyword{htest}
\keyword{multivariate}
\eof
\name{rcorr}
\alias{rcorr}
\title{Random correlation matrix}
\description{
Generates a random correlation matrix 
with the method of Marsaglia and Olkin (1984). 
}
\usage{
rcorr(d)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{d}{an integer > 0,  the order of the correlation matrix.}
}
\details{
  The algorithm uses \code{\link{rsphere}} to generate \eqn{d}
  vectors on a sphere in  \eqn{d}-space. If \eqn{Z}
  is a matrix with such vectors as rows, then the random
  correlation matrix is \eqn{ZZ'}.
}
\value{
a correlation matrix of order \code{d}.
}
\references{Marshall, G.\& Olkin, I. (1984).Generating correlation
  matrices. \emph{SIAM J. Sci. Stat. Comput.}, 5, 2, 470--475.}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{rsphere}}}
\examples{
## A random correlation matrix of order 3
rcorr(3)
## A random correlation matrix of order 5
rcorr(5)
}
\keyword{distribution}
\keyword{multivariate}
\eof
\name{rnormDag}
\alias{rnormDag}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Random sample from a decomposable Gaussian model}
\description{
  Generates a sample from a mean centered  multivariate normal
  distribution whose covariance matrix has a given triangular
  decomposition.
}
\usage{
rnormDag(n, A, Delta)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n}{an integer > 0, the sample size.}
  \item{A}{a square, upper triangular matrix with ones along the
    diagonal. It defines, together with \code{Delta},
    the concentration matrix (and also the covariance matrix)
    of the multivariate normal. The order of \code{A} is the
    number of components of the normal.}
  \item{Delta}{a numeric vector of length equal to the number of columns
    of \code{A}.}
}
\details{
  The value in position \eqn{(i,j)} of  \code{A} (with \eqn{i < j}) is
  a regression coefficient (with  sign changed) in the regression of
  variable \eqn{i} on variables \eqn{i+1, \dots, d}.

  The value in position \eqn{i} of \code{Delta} is the residual
  variance in the above regression.
  
}
\value{a matrix with \code{n} rows and \code{nrow(A)} columns,
  a sample from a multivariate normal distribution with mean zero
  and covariance matrix \code{S = solve(A) \%*\% diag(Delta) \%*\% t(solve(A))}.
}
\references{Cox, D. R. \& Wermuth, N. (1996). \emph{Multivariate
  dependencies}. London: Chapman \& Hall.}
\author{Giovanni M. Marchetti}

\seealso{\code{\link{triDec}}, \code{\link{fitDag}}}
\examples{
## Generate a sample of 100 observation from a multivariate normal
## The 
A <- matrix(
c(1, -2, -3,  0, 0,  0,  0,
  0,  1,  0, -4, 0,  0,  0,
  0,  0,  1,  2, 0,  0,  0,
  0,  0,  0,  1, 1, -5,  0,
  0,  0,  0,  0, 1,  0,  3,
  0,  0,  0,  0, 0,  1, -4,
  0,  0,  0,  0, 0,  0,  1), 7, 7, byrow=TRUE)
D <- rep(1, 7)
X <- rnormDag(100, A, D)

## The true covariance matrix
solve(A) \%*\% diag(D) \%*\% t(solve(A))

## Triangular decomposition of the sample covariance matrix
triDec(cov(X))$A
}
\keyword{distribution}
\keyword{multivariate}
\eof
\name{rsphere}
\alias{rsphere}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Random vectors on a sphere}
\description{
  Generates a sample of points uniformly distributed
  on the surface of a sphere in d-space.
}
\usage{
rsphere(n, d)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n}{an integer, the sample size.}
  \item{d}{an integer, the dimension of the space. For example, a circle
    is defined in 2D-space, a sphere in 3D-space.}
}
\details{The algorithm is based on normalising to length 1 each d-vector
   of a sample from a multivariate normal \eqn{N(0, I)}.
}
\value{a matrix of \code{n} rows and \code{d} columns.
}}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{rnorm}}, \code{\link{rcorr}}}
\examples{
## 100 points on circle
z <- rsphere(100,2)
plot(z)

## 100 points on a sphere
z <- rsphere(100, 3)
pairs(z)
}
\keyword{distribution}
\keyword{multivariate}


\eof
\name{shipley.test}
\alias{shipley.test}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Test of all independencies implied by a DAG}
\description{
  Computes a simultaneous test of all independence relationships
  implied by a given Gaussian model  defined according to
  a directed acyclic graph, based on the sample covariance matrix.
}
\usage{
shipley.test(S, n, A)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{S}{a symmetric positive definite matrix, the sample covariance matrix.}
  \item{n}{a positive integer, the sample size.}
  \item{A}{a square Boolean matrix, of the same dimension as \code{S},
    representing the edge matrix of a DAG.}
}
}
\details{
  The test statistic is \eqn{C = -2 \sum \ln p_j} where \eqn{p_j} are the
  p-values of tests of conditional independence in the basis set
  computed by \code{basiSet(A)}. The p-values are independent
  uniform variables on \eqn{(0,1)} and the statistic has exactly a
  chi square distribution on \eqn{2k} degrees of freedom  where
  \eqn{k} is the number of elements of the basis set.
  Shipley (2002) calls this test Fisher's C test.
}
\value{
  \item{ctest}{Test statistic \eqn{C}.}
  \item{df}{Degrees of freedom.}
  \item{pvalue}{The P-value of the test, assuming a two-sided alternative.}
}
\references{Shipley, B. (2000). A new inferential test
for path models based on directed acyclic graphs. \emph{Structural
  Equation Modeling}, 7(2), 206--218.}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{basiSet}}, \code{\link{pcor.test}}}
\examples{
## A decomposable model for the mathematics marks data
data(marks)
dag <- DAG(mec ~ vec+alg, vec ~ alg, sta ~ alg+ana, ana ~ alg)
shipley.test(cov(marks), n=88, dag)
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{swp}
\alias{swp}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Sweep operator}
\description{
Sweeps a covariance matrix with respect to a subset of indices. 
}
\usage{
swp(V, b)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{V}{a symmetric positive definite matrix, the covariance matrix.}
  \item{b}{a subset of indices of the columns of \code{V}.}
}
\details{
The sweep operator has been introduced by Beaton (1964) as a tool for
inverting symmetric matrices (cfr. Dempster, 1969).
}
\value{
  a square matrix \code{U} of the same order as \code{V}. If \code{a} is
  the complement of \code{b}, then \code{U[a,b]} is the matrix of
  regression coefficients of \code{a} given \code{b} and \code{U[a,a]}
  is the corresponding covariance matrix of the residuals.

  If \code{b} is empty the function returns \code{V}.

  If \code{b} is the vector \code{1:nrow(V)} (or its permutation) then
  the function returns the opposite of the inverse of \code{V}.
}
\references{
  Beaton, A.E. (1964). \emph{The use of special matrix operators
    in statistical calculus}. Ed.D. thesis, Harvard
  University. Reprinted as Educational Testing Service Research Bulletin
  64-51. Princeton.
  
  Dempster, A.P. (1969). \emph{Elements of continuous multivariate
    analysis}.  Reading: Addison-Wesley.

}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{fitDag}}}
\examples{
## A very simple example
V <- matrix(c(10, 1, 1, 2), 2, 2)
swp(V, 2)
}
\keyword{array}
\keyword{algebra}
\keyword{models}
\keyword{multivariate}
\eof
\name{topSort}
\alias{topSort}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Topological sort}
\description{
Permutates the nodes of a directed acyclic graph in such a way its edge
matrix is upper triangular.
}
\usage{
topSort(gmat)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gmat}{a square Boolean matrix with dimnames representing the
    edge matrix of a directed acyclic graph.}
}
\details{
The edge matrix of a directed acyclic graph (DAG) can always transformed
into an upper triangular matrix by a permutation applied both to rows
and columns.  The ordering needs not to be unique.
The algorithm to perform the ordering of the nodes is
called topological sort.

}
\value{
  \item{triu}{the sorted edge matrix in upper triangular form. If the
    edge matrix is already in upper triangular form it is leaved untrasformed.}
  \item{perm}{the vector of the permutation.}
}
\note{
  The order of the nodes defined by \code{DAG} is that of their first
  appearance in the model formulae (from left to right).
}
\references{
  Aho, A.V., Hopcrtoft, J.E. \& Ullman, J.D. (1983). \emph{Data structures
    and algorithms.} Reading: Addison-Wesley.
    
    Lauritzen, S. (1996). \emph{Graphical models}. Oxford:
Clarendon Press.
}
\author{Giovanni M. Marchetti} 
\seealso{\code{\link{DAG}}, \code{\link{is.acyclic}}}
\examples{
## A simple example
dag <- DAG(a ~ b, c ~ a + b, d ~ c + b)
dag
m <- topSort(dag)
m
dag[m$perm, m$perm]
}
\keyword{graphs}
\keyword{models}
\keyword{multivariate}
\eof
\name{triDec}
\alias{triDec}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Triangular decomposition of a covariance matrix}
\description{
  Decomposes a symmetric positive definite matrix with a variant of the
  Cholesky decomposition. 
}
\usage{
triDec(Sigma)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Sigma}{a symmetric positive definite matrix.}
}
\details{
  Any symmetric positive definite matrix \eqn{\Sigma} can be decomposed as
  \eqn{\Sigma = B \Delta B'} where \eqn{B} is upper triangular with ones
  along the main diagonal and \eqn{\Delta} is diagonal. If \eqn{\Sigma} is a covariance
  matrix the concentration matrix is \eqn{\Sigma^{-1} = A' \Delta^{-1} A} where
  \eqn{A = B^{-1}} is the matrix of the regression coefficients
  (with the sign changed) of a system of linear recursive regression
  equations  with independent residuals. In the equations each variable 
  \eqn{i} is regressed on the variables \eqn{i+1, \dots, d}.
  The elements on the diagonal of \eqn{\Delta} are the partial variances.}
}
\value{
  \item{A}{a square upper triangular matrix of the same order as
    \code{Sigma} with ones on the diagonal.}
  \item{B}{the inverse of \code{A}, another triangular matrix with unit diagonal.}
  \item{Delta}{a vector containing the diagonal values of \eqn{\Delta}.}
}
\references{Cox, D. R. \& Wermuth, N. (1996). \emph{Multivariate
  dependencies}. London: Chapman \& Hall.}
\author{Giovanni M. Marchetti}
\seealso{\code{\link{chol}}}
\examples{
## Triangular decomposition of a covariance matrix
B <- matrix(c(1,  -2, 0, 1,
              0,   1, 0, 1,
              0,   0, 1, 0,
              0,   0, 0, 1), 4, 4, byrow=TRUE)
B
D <- diag(c(3, 1, 2, 1))
S <- B \%*\% D \%*\% t(B)
triDec(S)
solve(B)
}
\keyword{array}
\keyword{algebra}
\keyword{models}
\keyword{multivariate}
\eof
